try:
    import pathlib
    pathlib2 = None
except ImportError:
    try:
        import pathlib2
        pathlib = pathlib2
    except ImportError:
        pathlib = None
        pathlib2 = None


try:
    from os import scandir
    use_scandir = True
    use_scandir_package = False
except ImportError:
    try:
        import scandir
        use_scandir = True
        use_scandir_package = True
    except ImportError:
        use_scandir = False
        use_scandir_package = False



"""Tests that a failed pytest properly displays the call stack.
Uses the output from running pytest with pytest_plugin_failing_test.py.
Regression test for #381.
"""
import pytest
import sys


@pytest.mark.skipif(sys.version_info[0] == 3 and sys.version_info[1] == 3,
                    reason='linecache uses fake fs in Python 3.3')
def test_failed_testresult_stacktrace():
    with open('testresult.txt') as f:
        contents = f.read()
    # before the fix, a triple question mark has been displayed
    # instead of the stacktrace
    assert contents
    print('contents', contents)
    assert '???' not in contents
    assert 'def test_fs(fs):' in contents



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Example for a test using a custom pytest fixture with an argument to Patcher
# Needs Python >= 3.6

import pytest

import pyfakefs.tests.pytest.example as example
from pyfakefs.fake_filesystem_unittest import Patcher


@pytest.mark.xfail
def test_example_file_failing(fs):
    """Test fails because EXAMPLE_FILE is cached in the module
    and not patched."""
    fs.create_file(example.EXAMPLE_FILE, contents='stuff here')
    check_that_example_file_is_in_fake_fs()


def test_example_file_passing_using_fixture(fs_reload_example):
    """Test passes if using a fixture that reloads the module containing
    EXAMPLE_FILE"""
    fs_reload_example.create_file(example.EXAMPLE_FILE, contents='stuff here')
    check_that_example_file_is_in_fake_fs()


def test_example_file_passing_using_patcher():
    """Test passes if using a Patcher instance that reloads the module
    containing EXAMPLE_FILE"""
    with Patcher(modules_to_reload=[example]) as patcher:
        patcher.fs.create_file(example.EXAMPLE_FILE, contents='stuff here')
        check_that_example_file_is_in_fake_fs()


def check_that_example_file_is_in_fake_fs():
    with open(example.EXAMPLE_FILE) as file:
        assert file.read() == 'stuff here'
    with example.EXAMPLE_FILE.open() as file:
        assert file.read() == 'stuff here'
    assert example.EXAMPLE_FILE.read_text() == 'stuff here'
    assert example.EXAMPLE_FILE.is_file()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unit tests for mox3_stubout."""

import datetime
import math
import os
import unittest
from os import path

from pyfakefs import mox3_stubout
from pyfakefs.tests import mox3_stubout_example


class NoPanicMath(object):
    real_math = math

    @staticmethod
    def fabs(_x):
        return 42

    def __getattr__(self, name):
        """Forwards any unfaked calls to the standard module."""
        return getattr(self.real_math, name)


class ExistingPath(object):
    real_path = path

    @staticmethod
    def exists(_path):
        return True

    def __getattr__(self, name):
        """Forwards any unfaked calls to the standard module."""
        return getattr(self.real_path, name)


class GroundhogDate(datetime.date):
    @classmethod
    def today(cls):
        return datetime.date(1993, 2, 2)


class StubOutForTestingTest(unittest.TestCase):
    def setUp(self):
        super(StubOutForTestingTest, self).setUp()
        self.stubber = mox3_stubout.StubOutForTesting()

    def test_stubout_method_with_set(self):
        non_existing_path = 'non_existing_path'
        self.assertFalse(
            mox3_stubout_example.check_if_exists(non_existing_path))
        self.stubber.set(os.path, 'exists', lambda x: True)
        self.assertTrue(
            mox3_stubout_example.check_if_exists(non_existing_path))
        self.stubber.unset_all()
        self.assertFalse(
            mox3_stubout_example.check_if_exists(non_existing_path))

    def test_stubout_class_with_set(self):
        self.assertGreater(mox3_stubout_example.tomorrow().year, 2000)

        self.stubber.set(datetime, 'date', GroundhogDate)
        self.assertEqual(mox3_stubout_example.tomorrow(),
                         datetime.date(1993, 2, 3))

        self.stubber.unset_all()
        self.assertGreater(mox3_stubout_example.tomorrow().year, 2000)

    def test_stubout_module_with_set(self):
        self.assertEqual(10, mox3_stubout_example.fabs(-10))

        self.stubber.set(mox3_stubout_example, 'math', NoPanicMath)
        self.assertEqual(42, mox3_stubout_example.fabs(-10))

        self.stubber.unset_all()
        self.assertEqual(10, mox3_stubout_example.fabs(-10))

    def test_set_raise_if_unknown_attribute(self):
        self.assertRaises(AttributeError, self.stubber.set,
                          os.path, 'exists_not', lambda x: True)
        self.assertRaises(AttributeError, self.stubber.set,
                          datetime, 'tomorrow', GroundhogDate)
        self.assertRaises(AttributeError, self.stubber.set,
                          mox3_stubout_example, 'math1', NoPanicMath)

    def test_stubout_method_with_smart_set(self):
        non_existing_path = 'non_existing_path'
        self.stubber.smart_set(os.path, 'exists', lambda x: True)
        self.assertTrue(
            mox3_stubout_example.check_if_exists(non_existing_path))
        self.stubber.smart_unset_all()
        self.assertFalse(
            mox3_stubout_example.check_if_exists(non_existing_path))

    def test_stubout_class_with_smart_set(self):
        self.stubber.smart_set(datetime, 'date', GroundhogDate)
        self.assertEqual(mox3_stubout_example.tomorrow(),
                         datetime.date(1993, 2, 3))

        self.stubber.smart_unset_all()
        self.assertGreater(mox3_stubout_example.tomorrow().year, 2000)

    def test_stubout_module_with_smart_set(self):
        self.assertEqual(10, mox3_stubout_example.fabs(-10))

        self.stubber.smart_set(mox3_stubout_example, 'math', NoPanicMath)
        self.assertEqual(42, mox3_stubout_example.fabs(-10))

        self.stubber.smart_unset_all()
        self.assertEqual(10, mox3_stubout_example.fabs(-10))

    def test_stubout_submodule_with_smart_set(self):
        # this one does not work with Set
        non_existing_path = 'non_existing_path'
        self.assertFalse(
            mox3_stubout_example.check_if_exists(non_existing_path))
        self.stubber.smart_set(os, 'path', ExistingPath)
        self.assertTrue(
            mox3_stubout_example.check_if_exists(non_existing_path))
        self.stubber.smart_unset_all()
        self.assertFalse(
            mox3_stubout_example.check_if_exists(non_existing_path))

    def test_smart_set_raise_if_unknown_attribute(self):
        self.assertRaises(AttributeError, self.stubber.smart_set,
                          os.path, 'exists_not', lambda x: True)
        self.assertRaises(AttributeError, self.stubber.smart_set,
                          datetime, 'tomorrow', GroundhogDate)
        self.assertRaises(AttributeError, self.stubber.smart_set,
                          mox3_stubout_example, 'math1', NoPanicMath)


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
#
# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A test suite that runs all tests for pyfakefs at once."""

import sys
import unittest

from pyfakefs.extra_packages import pathlib
from pyfakefs.tests import dynamic_patch_test, fake_stat_time_test
from pyfakefs.tests import fake_open_test
from pyfakefs.tests import fake_os_test
from pyfakefs.tests import example_test
from pyfakefs.tests import fake_filesystem_glob_test
from pyfakefs.tests import fake_filesystem_shutil_test
from pyfakefs.tests import fake_filesystem_test
from pyfakefs.tests import fake_filesystem_unittest_test
from pyfakefs.tests import fake_tempfile_test
from pyfakefs.tests import fake_filesystem_vs_real_test
from pyfakefs.tests import mox3_stubout_test

if pathlib:
    from pyfakefs.tests import fake_pathlib_test


class AllTests(unittest.TestSuite):
    """A test suite that runs all tests for pyfakefs at once."""

    def suite(self):  # pylint: disable-msg=C6409
        loader = unittest.defaultTestLoader
        self.addTests([
            loader.loadTestsFromModule(fake_filesystem_test),
            loader.loadTestsFromModule(fake_filesystem_glob_test),
            loader.loadTestsFromModule(fake_filesystem_shutil_test),
            loader.loadTestsFromModule(fake_os_test),
            loader.loadTestsFromModule(fake_stat_time_test),
            loader.loadTestsFromModule(fake_open_test),
            loader.loadTestsFromModule(fake_tempfile_test),
            loader.loadTestsFromModule(fake_filesystem_vs_real_test),
            loader.loadTestsFromModule(fake_filesystem_unittest_test),
            loader.loadTestsFromModule(example_test),
            loader.loadTestsFromModule(mox3_stubout_test),
            loader.loadTestsFromModule(dynamic_patch_test),
        ])
        if pathlib:
            self.addTests([
                loader.loadTestsFromModule(fake_pathlib_test)
            ])
        return self


if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(AllTests().suite())
    sys.exit(int(not result.wasSuccessful()))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example module that is used for testing modules that import file system modules
to be patched under another name.
"""
from os import path
from os.path import exists
import os as my_os
from os import stat

import sys

try:
    from pathlib import Path
except ImportError:
    try:
        from pathlib2 import Path
    except ImportError:
        Path = None


def check_if_exists1(filepath):
    # test patching module imported under other name
    return my_os.path.exists(filepath)


def check_if_exists2(filepath):
    # tests patching path imported from os
    return path.exists(filepath)


if Path:
    def check_if_exists3(filepath):
        # tests patching Path imported from pathlib
        return Path(filepath).exists()


def check_if_exists4(filepath, file_exists=my_os.path.exists):
    # this is a similar case as in the tempfile implementation under Posix
    return file_exists(filepath)


def check_if_exists5(filepath):
    # tests patching `exists` imported from os.path
    return exists(filepath)


def file_stat(filepath):
    # tests patching `stat` imported from os
    return stat(filepath)


def system_stat(filepath):
    if sys.platform == 'win32':
        from nt import stat as system_stat
    else:
        from posix import stat as system_stat
    return system_stat(filepath)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helper classes use for fake file system implementation."""
import io
import locale
import sys
from copy import copy
from stat import S_IFLNK

import os

IS_PY2 = sys.version_info[0] < 3


try:
    text_type = unicode  # Python 2
except NameError:
    text_type = str      # Python 3


def is_int_type(val):
    """Return True if `val` is of integer type."""
    try:               # Python 2
        return isinstance(val, (int, long))
    except NameError:  # Python 3
        return isinstance(val, int)


def is_byte_string(val):
    """Return True if `val` is a bytes-like object, False for a unicode
    string."""
    if not IS_PY2:
        return not hasattr(val, 'encode')
    return isinstance(val, (memoryview, str))


def is_unicode_string(val):
    """Return True if `val` is a unicode string, False for a bytes-like
    object."""
    try:               # Python 2
        return isinstance(val, unicode)
    except NameError:  # Python 3
        return hasattr(val, 'encode')


def make_string_path(dir_name):
    if sys.version_info >= (3, 6):
        dir_name = os.fspath(dir_name)
    return dir_name


class FakeStatResult(object):
    """Mimics os.stat_result for use as return type of `stat()` and similar.
    This is needed as `os.stat_result` has no possibility to set
    nanosecond times directly.
    """
    try:
        long_type = long  # Python 2
    except NameError:
        long_type = int   # Python 3
    _stat_float_times = sys.version_info >= (2, 5)

    def __init__(self, is_windows, user_id, group_id, initial_time=None):
        self._use_float = None
        self.st_mode = None
        self.st_ino = None
        self.st_dev = None
        self.st_nlink = 0
        self.st_uid = user_id
        self.st_gid = group_id
        self._st_size = None
        self.is_windows = is_windows
        if initial_time is not None:
            self._st_atime_ns = self.long_type(initial_time * 1e9)
        else:
            self._st_atime_ns = None
        self._st_mtime_ns = self._st_atime_ns
        self._st_ctime_ns = self._st_atime_ns

    @property
    def use_float(self):
        if self._use_float is None:
            return self.stat_float_times()
        return self._use_float

    @use_float.setter
    def use_float(self, val):
        self._use_float = val

    def __eq__(self, other):
        return (
            isinstance(other, FakeStatResult) and
            self._st_atime_ns == other._st_atime_ns and
            self._st_ctime_ns == other._st_ctime_ns and
            self._st_mtime_ns == other._st_mtime_ns and
            self.st_size == other.st_size and
            self.st_gid == other.st_gid and
            self.st_uid == other.st_uid and
            self.st_nlink == other.st_nlink and
            self.st_dev == other.st_dev and
            self.st_ino == other.st_ino and
            self.st_mode == other.st_mode
        )

    def __ne__(self, other):
        return not self == other

    def copy(self):
        """Return a copy where the float usage is hard-coded to mimic the
        behavior of the real os.stat_result.
        """
        stat_result = copy(self)
        stat_result.use_float = self.use_float
        return stat_result

    def set_from_stat_result(self, stat_result):
        """Set values from a real os.stat_result.
        Note: values that are controlled by the fake filesystem are not set.
        This includes st_ino, st_dev and st_nlink.
        """
        self.st_mode = stat_result.st_mode
        self.st_uid = stat_result.st_uid
        self.st_gid = stat_result.st_gid
        self._st_size = stat_result.st_size
        if sys.version_info < (3, 3):
            self._st_atime_ns = self.long_type(stat_result.st_atime * 1e9)
            self._st_mtime_ns = self.long_type(stat_result.st_mtime * 1e9)
            self._st_ctime_ns = self.long_type(stat_result.st_ctime * 1e9)
        else:
            self._st_atime_ns = stat_result.st_atime_ns
            self._st_mtime_ns = stat_result.st_mtime_ns
            self._st_ctime_ns = stat_result.st_ctime_ns

    @classmethod
    def stat_float_times(cls, newvalue=None):
        """Determine whether a file's time stamps are reported as floats
        or ints.

        Calling without arguments returns the current value.
        The value is shared by all instances of FakeOsModule.

        Args:
            newvalue: If `True`, mtime, ctime, atime are reported as floats.
                Otherwise, they are returned as ints (rounding down).
        """
        if newvalue is not None:
            cls._stat_float_times = bool(newvalue)
        return cls._stat_float_times

    @property
    def st_ctime(self):
        """Return the creation time in seconds."""
        ctime = self._st_ctime_ns / 1e9
        return ctime if self.use_float else int(ctime)

    @property
    def st_atime(self):
        """Return the access time in seconds."""
        atime = self._st_atime_ns / 1e9
        return atime if self.use_float else int(atime)

    @property
    def st_mtime(self):
        """Return the modification time in seconds."""
        mtime = self._st_mtime_ns / 1e9
        return mtime if self.use_float else int(mtime)

    @st_ctime.setter
    def st_ctime(self, val):
        """Set the creation time in seconds."""
        self._st_ctime_ns = self.long_type(val * 1e9)

    @st_atime.setter
    def st_atime(self, val):
        """Set the access time in seconds."""
        self._st_atime_ns = self.long_type(val * 1e9)

    @st_mtime.setter
    def st_mtime(self, val):
        """Set the modification time in seconds."""
        self._st_mtime_ns = self.long_type(val * 1e9)

    @property
    def st_size(self):
        if self.st_mode & S_IFLNK == S_IFLNK and self.is_windows:
            return 0
        return self._st_size

    @st_size.setter
    def st_size(self, val):
        self._st_size = val

    def __getitem__(self, item):
        """Implement item access to mimic `os.stat_result` behavior."""
        import stat

        if item == stat.ST_MODE:
            return self.st_mode
        if item == stat.ST_INO:
            return self.st_ino
        if item == stat.ST_DEV:
            return self.st_dev
        if item == stat.ST_NLINK:
            return self.st_nlink
        if item == stat.ST_UID:
            return self.st_uid
        if item == stat.ST_GID:
            return self.st_gid
        if item == stat.ST_SIZE:
            return self.st_size
        if item == stat.ST_ATIME:
            # item access always returns int for backward compatibility
            return int(self.st_atime)
        if item == stat.ST_MTIME:
            return int(self.st_mtime)
        if item == stat.ST_CTIME:
            return int(self.st_ctime)
        raise ValueError('Invalid item')

    if sys.version_info >= (3, 3):
        @property
        def st_atime_ns(self):
            """Return the access time in nanoseconds."""
            return self._st_atime_ns

        @property
        def st_mtime_ns(self):
            """Return the modification time in nanoseconds."""
            return self._st_mtime_ns

        @property
        def st_ctime_ns(self):
            """Return the creation time in nanoseconds."""
            return self._st_ctime_ns

        @st_atime_ns.setter
        def st_atime_ns(self, val):
            """Set the access time in nanoseconds."""
            self._st_atime_ns = val

        @st_mtime_ns.setter
        def st_mtime_ns(self, val):
            """Set the modification time of the fake file in nanoseconds."""
            self._st_mtime_ns = val

        @st_ctime_ns.setter
        def st_ctime_ns(self, val):
            """Set the creation time of the fake file in nanoseconds."""
            self._st_ctime_ns = val


class FileBufferIO(object):
    """Stream class that handles both Python2 and Python3 string and
    byte contents for files. The standard io.StringIO cannot be used
    for strings due to the slightly different handling of newline mode.
    Uses an io.BytesIO stream for the raw data and adds handling of encoding
    and newlines.
    """
    def __init__(self, contents=None, linesep='\n', binary=False,
                 newline=None, encoding=None, errors='strict'):
        self._newline = newline
        self._encoding = encoding
        self.errors = errors
        self._linesep = linesep
        self.binary = binary
        self._bytestream = io.BytesIO()
        if contents is not None:
            self.putvalue(contents)
            self._bytestream.seek(0)

    def encoding(self):
        return self._encoding or locale.getpreferredencoding(False)

    def encoded_string(self, contents):
        if is_byte_string(contents):
            return contents
        return contents.encode(self.encoding(), self.errors)

    def decoded_string(self, contents):
        if IS_PY2 and not self._encoding:
            return contents
        return contents.decode(self.encoding(), self.errors)

    def convert_newlines_for_writing(self, s):
        if self.binary:
            return s
        if self._newline in (None, '-'):
            return s.replace('\n', self._linesep)
        if self._newline in ('', '\n'):
            return s
        return s.replace('\n', self._newline)

    def convert_newlines_after_reading(self, s):
        if self._newline is None:
            return s.replace('\r\n', '\n').replace('\r', '\n')
        if self._newline == '-':
            return s.replace(self._linesep, '\n')
        return s

    def read(self, size=-1):
        contents = self._bytestream.read(size)
        if self.binary:
            return contents
        return self.convert_newlines_after_reading(
            self.decoded_string(contents))

    def readline(self, size=-1):
        seek_pos = self._bytestream.tell()
        byte_contents = self._bytestream.read(size)
        if self.binary:
            read_contents = byte_contents
            LF = b'\n'
        else:
            read_contents = self.convert_newlines_after_reading(
                self.decoded_string(byte_contents))
            LF = '\n'
        end_pos = 0

        if self._newline is None:
            end_pos = self._linelen_for_universal_newlines(byte_contents)
            if end_pos > 0:
                length = read_contents.find(LF) + 1
        elif self._newline == '':
            end_pos = self._linelen_for_universal_newlines(byte_contents)
            if end_pos > 0:
                if byte_contents[end_pos - 1] == ord(b'\r'):
                    newline = '\r'
                elif end_pos > 1 and byte_contents[end_pos - 2] == ord(b'\r'):
                    newline = '\r\n'
                else:
                    newline = '\n'
                length = read_contents.find(newline) + len(newline)
        else:
            newline = '\n' if self._newline == '-' else self._newline
            length = read_contents.find(newline)
            if length >= 0:
                nl_len = len(newline)
                end_pos = byte_contents.find(newline.encode()) + nl_len
                length += nl_len

        if end_pos == 0:
            length = len(read_contents)
            end_pos = len(byte_contents)

        self._bytestream.seek(seek_pos + end_pos)
        return (byte_contents[:end_pos] if self.binary
                else read_contents[:length])

    def _linelen_for_universal_newlines(self, byte_contents):
        if self.binary:
            return byte_contents.find(b'\n') + 1
        pos_lf = byte_contents.find(b'\n')
        pos_cr = byte_contents.find(b'\r')
        if pos_lf == -1 and pos_cr == -1:
            return 0
        if pos_lf != -1 and (pos_lf < pos_cr or pos_cr == -1):
            end_pos = pos_lf
        else:
            end_pos = pos_cr
        if end_pos == pos_cr and end_pos + 1 == pos_lf:
            end_pos = pos_lf
        return end_pos + 1

    def readlines(self, size=-1):
        remaining_size = size
        lines = []
        while True:
            line = self.readline(remaining_size)
            if not line:
                return lines
            lines.append(line)
            if size > 0:
                remaining_size -= len(line)
                if remaining_size <= 0:
                    return lines

    def putvalue(self, s):
        self._bytestream.write(self.encoded_string(s))

    def write(self, s):
        if not IS_PY2 and self.binary != is_byte_string(s):
            raise TypeError('Incorrect type for writing')
        contents = self.convert_newlines_for_writing(s)
        length = len(contents)
        self.putvalue(contents)
        return length

    def writelines(self, lines):
        for line in lines:
            self.write(line)

    def __iter__(self):
        return self

    def __next__(self):
        line = self.readline()
        if not line:
            raise StopIteration
        return line

    # Python 2 version
    def next(self):
        return self.__next__()

    def __getattr__(self, name):
        return getattr(self._bytestream, name)


class NullFileBufferIO(FileBufferIO):
    """Special stream for null device. Does nothing on writing."""
    def putvalue(self, s):
        pass



# Copyright 2014 Altera Corporation. All Rights Reserved.
# Copyright 2015-2017 John McGehee
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module provides a base class derived from `unittest.TestClass`
for unit tests using the :py:class:`pyfakefs` module.

`fake_filesystem_unittest.TestCase` searches `sys.modules` for modules
that import the `os`, `io`, `path` `shutil`, and `pathlib` modules.

The `setUpPyfakefs()` method binds these modules to the corresponding fake
modules from `pyfakefs`.  Further, the `open()` built-in is bound to a fake
`open()`.  In Python 2, built-in `file()` is similarly bound to the fake
`open()`.

It is expected that `setUpPyfakefs()` be invoked at the beginning of the
derived class' `setUp()` method.  There is no need to add anything to the
derived class' `tearDown()` method.

During the test, everything uses the fake file system and modules.  This means
that even in your test fixture, familiar functions like `open()` and
`os.makedirs()` manipulate the fake file system.

Existing unit tests that use the real file system can be retrofitted to use
pyfakefs by simply changing their base class from `:py:class`unittest.TestCase`
to `:py:class`pyfakefs.fake_filesystem_unittest.TestCase`.
"""
import doctest
import inspect
import sys
import tempfile
import unittest
import zipfile  # import here to make sure it gets correctly stubbed, see #427

from pyfakefs.deprecator import Deprecator

try:
    from importlib.machinery import ModuleSpec
except ImportError:
    ModuleSpec = object

try:
    # python >= 3.4
    from importlib import reload
except ImportError:
    try:
        # python 3.0 - 3.3
        from imp import reload
    except ImportError:
        # python 2 - reload is built-in
        pass

from pyfakefs import fake_filesystem
from pyfakefs import fake_filesystem_shutil
from pyfakefs import mox3_stubout
from pyfakefs.extra_packages import pathlib, pathlib2, use_scandir

if pathlib:
    from pyfakefs import fake_pathlib

if use_scandir:
    from pyfakefs import fake_scandir

if sys.version_info < (3, ):
    import __builtin__ as builtins  # pylint: disable=import-error
else:
    import builtins

OS_MODULE = 'nt' if sys.platform == 'win32' else 'posix'
PATH_MODULE = 'ntpath' if sys.platform == 'win32' else 'posixpath'

def load_doctests(loader, tests, ignore, module,
                  additional_skip_names=None):  # pylint: disable=unused-argument
    """Load the doctest tests for the specified module into unittest.
        Args:
            loader, tests, ignore : arguments passed in from `load_tests()`
            module: module under test
            additional_skip_names: see :py:class:`TestCase` for an explanation

    File `example_test.py` in the pyfakefs release provides a usage example.
    """
    _patcher = Patcher(additional_skip_names=additional_skip_names)
    globs = _patcher.replace_globs(vars(module))
    tests.addTests(doctest.DocTestSuite(module,
                                        globs=globs,
                                        setUp=_patcher.setUp,
                                        tearDown=_patcher.tearDown))
    return tests


class TestCaseMixin(object):
    """Test case mixin that automatically replaces file-system related
    modules by fake implementations.

    Attributes:
        additional_skip_names: names of modules inside of which no module
            replacement shall be performed, in addition to the names in
            :py:attr:`fake_filesystem_unittest.Patcher.SKIPNAMES`.
        modules_to_reload: A list of modules that need to be reloaded
            to be patched dynamically; may be needed if the module
            imports file system modules under an alias

            .. note:: This is done independently of `use_dynamic_patch`

            .. caution:: Reloading modules may have unwanted side effects.
        use_dynamic_patch: If `True`, dynamic patching after setup is used
            (for example for modules loaded locally inside of functions).
            Can be switched off if it causes unwanted side effects.
        modules_to_patch: A dictionary of fake modules mapped to the
            patched module names. Can be used to add patching of modules
            not provided by `pyfakefs`.
            If you want to patch a class in a module imported using
            `from some_module import SomeClass`, you have to specify
            `some_module.Class` as the key for the fake class.

    If you specify the attribute `additional_skip_names` here
    and you have DocTests, consider also specifying the same argument to
    :py:func:`load_doctests`.

    Example usage in derived test classes::

        from unittest import TestCase
        from fake_filesystem_unittest import TestCaseMixin

        class MyTestCase(TestCase, TestCaseMixin):
            def __init__(self, methodName='runTest'):
                super(MyTestCase, self).__init__(
                    methodName=methodName,
                    additional_skip_names=['posixpath'])

        import sut

        class AnotherTestCase(TestCase, TestCaseMixin):
            def __init__(self, methodName='runTest'):
                super(MyTestCase, self).__init__(
                    methodName=methodName, modules_to_reload=[sut])
    """

    additional_skip_names = None
    modules_to_reload = None
    use_dynamic_patch = True
    modules_to_patch = None

    @property
    def fs(self):
        return self._stubber.fs

    def setUpPyfakefs(self,
                      additional_skip_names=None,
                      use_dynamic_patch=None,
                      modules_to_reload=None,
                      modules_to_patch=None):
        """Bind the file-related modules to the :py:class:`pyfakefs` fake file
        system instead of the real file system.  Also bind the fake `open()`
        function, and on Python 2, the `file()` function.

        Invoke this at the beginning of the `setUp()` method in your unit test
        class.
        For the arguments, see the `TestCaseMixin` attribute description.
        If any of the arguments is not None, it overwrites the settings for
        the current test case. Settings the arguments here may be a more
        convenient way to adapt the setting than overwriting `__init__()`.
        """
        if additional_skip_names is None:
            additional_skip_names = self.additional_skip_names
        if use_dynamic_patch is None:
            use_dynamic_patch = self.use_dynamic_patch
        if modules_to_reload is None:
            modules_to_reload = self.modules_to_reload
        if modules_to_patch is None:
            modules_to_patch = self.modules_to_patch
        self._stubber = Patcher(
            additional_skip_names=additional_skip_names,
            use_dynamic_patch=use_dynamic_patch,
            modules_to_reload=modules_to_reload,
            modules_to_patch=modules_to_patch)

        self._stubber.setUp()
        self.addCleanup(self._stubber.tearDown)

    def pause(self):
        """Pause the patching of the file system modules until `resume` is
        called. After that call, all file system calls are executed in the
        real file system.
        Calling pause() twice is silently ignored.

        """
        self._stubber.pause()

    def resume(self):
        """Resume the patching of the file system modules if `pause` has
        been called before. After that call, all file system calls are
        executed in the fake file system.
        Does nothing if patching is not paused.
        """
        self._stubber.resume()


class TestCase(unittest.TestCase, TestCaseMixin):
    """Test case class that automatically replaces file-system related
    modules by fake implementations.
    """

    def __init__(self, methodName='runTest',
                 additional_skip_names=None,
                 modules_to_reload=None,
                 use_dynamic_patch=True,
                 modules_to_patch=None):
        """Creates the test class instance and the stubber used to stub out
        file system related modules.

        Args:
            methodName: The name of the test method (same as in
                unittest.TestCase)
        """
        super(TestCase, self).__init__(methodName)

        self.additional_skip_names = additional_skip_names
        self.modules_to_reload = modules_to_reload
        self.use_dynamic_patch = use_dynamic_patch
        self.modules_to_patch = modules_to_patch

    @Deprecator('add_real_file')
    def copyRealFile(self, real_file_path, fake_file_path=None,
                     create_missing_dirs=True):
        """Add the file `real_file_path` in the real file system to the same
        path in the fake file system.

        **This method is deprecated** in favor of
        :py:meth:`FakeFilesystem..add_real_file`.
        `copyRealFile()` is retained with limited functionality for backward
        compatibility only.

        Args:
          real_file_path: Path to the file in both the real and fake
            file systems
          fake_file_path: Deprecated.  Use the default, which is
            `real_file_path`.
            If a value other than `real_file_path` is specified, a `ValueError`
            exception will be raised.
          create_missing_dirs: Deprecated.  Use the default, which creates
            missing directories in the fake file system.  If `False` is
            specified, a `ValueError` exception is raised.

        Returns:
          The newly created FakeFile object.

        Raises:
          IOError: If the file already exists in the fake file system.
          ValueError: If deprecated argument values are specified.

        See:
          :py:meth:`FakeFileSystem.add_real_file`
        """
        if fake_file_path is not None and real_file_path != fake_file_path:
            raise ValueError("CopyRealFile() is deprecated and no longer "
                             "supports different real and fake file paths")
        if not create_missing_dirs:
            raise ValueError("CopyRealFile() is deprecated and no longer "
                             "supports NOT creating missing directories")
        return self._stubber.fs.add_real_file(real_file_path, read_only=False)

    @DeprecationWarning
    def tearDownPyfakefs(self):
        """This method is deprecated and exists only for backward
        compatibility. It does nothing.
        """
        pass


class Patcher(object):
    """
    Instantiate a stub creator to bind and un-bind the file-related modules to
    the :py:mod:`pyfakefs` fake modules.

    The arguments are explained in :py:class:`TestCase`.

    :py:class:`Patcher` is used in :py:class:`TestCase`.  :py:class:`Patcher`
    also works as a context manager for PyTest::

        with Patcher():
            doStuff()
    """
    SKIPMODULES = {None, fake_filesystem, fake_filesystem_shutil, sys}
    '''Stub nothing that is imported within these modules.
    `sys` is included to prevent `sys.path` from being stubbed with the fake
    `os.path`.
    '''
    assert None in SKIPMODULES, ("sys.modules contains 'None' values;"
                                 " must skip them.")

    IS_WINDOWS = sys.platform in ('win32', 'cygwin')

    SKIPNAMES = {'os', 'path', 'io', 'genericpath', OS_MODULE, PATH_MODULE}
    if pathlib:
        SKIPNAMES.add('pathlib')

    def __init__(self, additional_skip_names=None,
                 modules_to_reload=None, use_dynamic_patch=True,
                 modules_to_patch=None):
        """For a description of the arguments, see TestCase.__init__"""

        self._skipNames = self.SKIPNAMES.copy()

        if additional_skip_names is not None:
            self._skipNames.update(additional_skip_names)

        self.modules_to_reload = [tempfile]
        if modules_to_reload is not None:
            self.modules_to_reload.extend(modules_to_reload)
        self._use_dynamic_patch = use_dynamic_patch

        # Attributes set by _findModules()

        # IMPORTANT TESTING NOTE: Whenever you add a new module below, test
        # it by adding an attribute in fixtures/module_with_attributes.py
        # and a test in fake_filesystem_unittest_test.py, class
        # TestAttributesWithFakeModuleNames.
        self._fake_module_classes = {
            'os': fake_filesystem.FakeOsModule,
            'shutil': fake_filesystem_shutil.FakeShutilModule,
            'io': fake_filesystem.FakeIoModule,
        }

        # class modules maps class names against a list of modules they can
        # be contained in - this allows for alternative modules like
        # `pathlib` and `pathlib2`
        self._class_modules = {}
        if pathlib:
            self._fake_module_classes[
                'pathlib'] = fake_pathlib.FakePathlibModule
            self._fake_module_classes[
                'Path'] = fake_pathlib.FakePathlibPathModule
            mod_name = 'pathlib2' if pathlib2 is not None else 'pathlib'
            self._class_modules['Path'] = [mod_name]
        if use_scandir:
            self._fake_module_classes[
                'scandir'] = fake_scandir.FakeScanDirModule

        if modules_to_patch is not None:
            for name, fake_module in modules_to_patch.items():
                self._fake_module_classes[name] = fake_module

        # handle patching function imported separately like
        # `from os import stat`
        # each patched function name has to be looked up separately
        self._fake_module_functions = {}
        for mod_name, fake_module in self._fake_module_classes.items():
            modnames = (
                (mod_name, OS_MODULE) if mod_name == 'os' else (mod_name,)
            )
            if (hasattr(fake_module, 'dir') and
                    inspect.isfunction(fake_module.dir)):
                for fct_name in fake_module.dir():
                    self._fake_module_functions[fct_name] = (
                        modnames,
                        getattr(fake_module, fct_name),
                        mod_name
                    )
        # special handling for functions in os.path
        fake_module = fake_filesystem.FakePathModule
        for fct_name in fake_module.dir():
            self._fake_module_functions[fct_name] = (
                ('genericpath', PATH_MODULE),
                getattr(fake_module, fct_name),
                PATH_MODULE
            )

        # Attributes set by _refresh()
        self._modules = {}
        self._fct_modules = {}
        self._stubs = None
        self.fs = None
        self.fake_open = None
        self.fake_modules = {}
        self._dyn_patcher = None

        # _isStale is set by tearDown(), reset by _refresh()
        self._isStale = True
        self._patching = False

    def __enter__(self):
        """Context manager for usage outside of
        fake_filesystem_unittest.TestCase.
        Ensure that all patched modules are removed in case of an
        unhandled exception.
        """
        self.setUp()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.tearDown()

    def _find_modules(self):
        """Find and cache all modules that import file system modules.
        Later, `setUp()` will stub these with the fake file system
        modules.
        """
        def is_fct(module, name):
            fct = module.__dict__.get(name)
            return (fct is not None and
                    (inspect.isfunction(fct) or inspect.isbuiltin(fct)) and
                    fct.__module__ in self._fake_module_functions[name][0])

        module_names = list(self._fake_module_classes.keys()) + [PATH_MODULE]
        for name, module in set(sys.modules.items()):
            try:
                if (module in self.SKIPMODULES or
                        not inspect.ismodule(module) or
                        module.__name__.split('.')[0] in self._skipNames):
                    continue
            except AttributeError:
                # workaround for some py (part of pytest) versions
                # where py.error has no __name__ attribute
                # see https://github.com/pytest-dev/py/issues/73
                continue

            modules = {name: mod for name, mod in module.__dict__.items()
                       if inspect.ismodule(mod) and
                       mod.__name__ in module_names
                       or inspect.isclass(mod) and
                       mod.__module__ in self._class_modules.get(name, [])}
            for name, mod in modules.items():
                self._modules.setdefault(name, set()).add((module,
                                                           mod.__name__))

            functions = [name for name in self._fake_module_functions
                         if is_fct(module, name)]
            for name in functions:
                self._fct_modules.setdefault(name, set()).add(module)

    def _refresh(self):
        """Renew the fake file system and set the _isStale flag to `False`."""
        if self._stubs is not None:
            self._stubs.smart_unset_all()
        self._stubs = mox3_stubout.StubOutForTesting()

        self.fs = fake_filesystem.FakeFilesystem(patcher=self)
        for name in self._fake_module_classes:
            self.fake_modules[name] = self._fake_module_classes[name](self.fs)
        self.fake_modules[PATH_MODULE] = self.fake_modules['os'].path
        self.fake_open = fake_filesystem.FakeFileOpen(self.fs)

        self._isStale = False

    def setUp(self, doctester=None):
        """Bind the file-related modules to the :py:mod:`pyfakefs` fake
        modules real ones.  Also bind the fake `file()` and `open()` functions.
        """
        temp_dir = tempfile.gettempdir()
        self._find_modules()
        self._refresh()

        if doctester is not None:
            doctester.globs = self.replace_globs(doctester.globs)

        self.start_patching()

        # the temp directory is assumed to exist at least in `tempfile1`,
        # so we create it here for convenience
        self.fs.create_dir(temp_dir)

    def start_patching(self):
        if not self._patching:
            self._patching = True
            if sys.version_info < (3,):
                # file() was eliminated in Python3
                self._stubs.smart_set(builtins, 'file', self.fake_open)
            self._stubs.smart_set(builtins, 'open', self.fake_open)
            for name, modules in self._modules.items():
                for module, attr in modules:
                    self._stubs.smart_set(module, name, self.fake_modules[attr])
            for name, modules in self._fct_modules.items():
                _, method, mod_name = self._fake_module_functions[name]
                fake_module = self.fake_modules[mod_name]
                attr = method.__get__(fake_module, fake_module.__class__)
                for module in modules:
                    self._stubs.smart_set(module, name, attr)
            self._dyn_patcher = DynamicPatcher(self)
            sys.meta_path.insert(0, self._dyn_patcher)
            for module in self.modules_to_reload:
                if module.__name__ in sys.modules:
                    reload(module)
            if not self._use_dynamic_patch:
                self._dyn_patcher.cleanup()
                sys.meta_path.pop(0)

    def replace_globs(self, globs_):
        globs = globs_.copy()
        if self._isStale:
            self._refresh()
        for name in self._fake_module_classes:
            if name in globs:
                globs[name] = self._fake_module_classes[name](self.fs)
        return globs

    def tearDown(self, doctester=None):
        """Clear the fake filesystem bindings created by `setUp()`."""
        self.stop_patching()

    def stop_patching(self):
        if self._patching:
            self._isStale = True
            self._patching = False
            self._stubs.smart_unset_all()
            if self._use_dynamic_patch:
                self._dyn_patcher.cleanup()
                sys.meta_path.pop(0)

    def pause(self):
        """Pause the patching of the file system modules until `resume` is
        called. After that call, all file system calls are executed in the
        real file system.
        Calling pause() twice is silently ignored.

        """
        self.stop_patching()

    def resume(self):
        """Resume the patching of the file system modules if `pause` has
        been called before. After that call, all file system calls are
        executed in the fake file system.
        Does nothing if patching is not paused.
        """
        self.start_patching()


class DynamicPatcher(object):
    """A file loader that replaces file system related modules by their
    fake implementation if they are loaded after calling `setupPyFakefs()`.
    Implements the protocol needed for import hooks.
    """

    def __init__(self, patcher):
        self._patcher = patcher
        self.sysmodules = {}
        self.modules = self._patcher.fake_modules

        # remove all modules that have to be patched from `sys.modules`,
        # otherwise the find_... methods will not be called
        for name in self.modules:
            if self.needs_patch(name) and name in sys.modules:
                self.sysmodules[name] = sys.modules[name]
                del sys.modules[name]

        for name, module in self.modules.items():
            sys.modules[name] = module

    def cleanup(self):
        for module in self.sysmodules:
            sys.modules[module] = self.sysmodules[module]
        for module in self._patcher.modules_to_reload:
            if module.__name__ in sys.modules:
                reload(module)

    def needs_patch(self, name):
        """Check if the module with the given name shall be replaced."""
        if name not in self.modules:
            return False
        if (name in sys.modules and
                type(sys.modules[name]) == self.modules[name]):
            return False
        return True

    def find_spec(self, fullname, path, target=None):
        """Module finder for Python 3."""
        if self.needs_patch(fullname):
            return ModuleSpec(fullname, self)

    def find_module(self, fullname, path=None):
        """Module finder for Python 2."""
        if self.needs_patch(fullname):
            return self

    def load_module(self, fullname):
        """Replaces the module by its fake implementation."""
        sys.modules[fullname] = self.modules[fullname]
        return self.modules[fullname]



import sys
import unittest

from pyfakefs import extra_packages

if extra_packages.pathlib2:
    extra_packages.pathlib = None
    extra_packages.pathlib2 = None

if extra_packages.use_scandir_package:
    extra_packages.use_scandir = False
    extra_packages.use_scandir_package = False

from pyfakefs.tests.all_tests import AllTests


if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(AllTests().suite())
    sys.exit(int(not result.wasSuccessful()))



# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A fake shutil module implementation that uses fake_filesystem for
unit tests.
Note that only `shutildisk_usage()` is faked, the rest of the functions shall
work fine with the fake file system if `os`/`os.path` are patched.

:Includes:
  FakeShutil: Uses a FakeFilesystem to provide a fake replacement for the
    shutil module.

:Usage:
  The fake implementation is automatically involved if using
  `fake_filesystem_unittest.TestCase`, pytest fs fixture,
  or directly `Patcher`.
"""

import shutil
import sys

from pyfakefs.helpers import IS_PY2


class FakeShutilModule(object):
    """Uses a FakeFilesystem to provide a fake replacement for shutil module.
    """

    @staticmethod
    def dir():
        """Return the list of patched function names. Used for patching
        functions imported from the module.
        """
        if not IS_PY2:
            return 'disk_usage',
        return ()

    def __init__(self, filesystem):
        """Construct fake shutil module using the fake filesystem.

        Args:
          filesystem:  FakeFilesystem used to provide file system information
        """
        self.filesystem = filesystem
        self._shutil_module = shutil

    if sys.version_info >= (3, 3):
        def disk_usage(self, path):
            """Return the total, used and free disk space in bytes as named tuple
            or placeholder holder values simulating unlimited space if not set.
            New in Python 3.3.

            Args:
              path: defines the filesystem device which is queried
            """
            return self.filesystem.get_disk_usage(path)

    def __getattr__(self, name):
        """Forwards any non-faked calls to the standard shutil module."""
        return getattr(self._shutil_module, name)



"""A pytest plugin for using pyfakefs as a fixture

When pyfakefs is installed, the "fs" fixture becomes available.

:Usage:

def my_fakefs_test(fs):
    fs.create_file('/var/data/xx1.txt')
    assert os.path.exists('/var/data/xx1.txt')
"""

import linecache
import sys

import py
import pytest

from pyfakefs.fake_filesystem_unittest import Patcher

if sys.version_info >= (3,):
    import builtins
else:
    import __builtin__ as builtins

Patcher.SKIPMODULES.add(py)  # Ignore pytest components when faking filesystem

# The "linecache" module is used to read the test file in case of test failure
# to get traceback information before test tear down.
# In order to make sure that reading the test file is not faked,
# we both skip faking the module, and add the build-in open() function
# as a local function in the module
Patcher.SKIPMODULES.add(linecache)
linecache.open = builtins.open


@pytest.fixture
def fs(request):
    """ Fake filesystem. """
    patcher = Patcher()
    patcher.setUp()
    request.addfinalizer(patcher.tearDown)
    return patcher.fs



#! /usr/bin/env python
#
# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test for glob using fake_filesystem."""

import glob
import os
import unittest

from pyfakefs import fake_filesystem_unittest


class FakeGlobUnitTest(fake_filesystem_unittest.TestCase):
    def setUp(self):
        self.setUpPyfakefs()
        directory = './xyzzy'
        self.fs.create_dir(directory)
        self.fs.create_dir('%s/subdir' % directory)
        self.fs.create_dir('%s/subdir2' % directory)
        self.fs.create_file('%s/subfile' % directory)
        self.fs.create_file('[Temp]')

    def test_glob_empty(self):
        self.assertEqual(glob.glob(''), [])

    def test_glob_star(self):
        basedir = os.sep + 'xyzzy'
        self.assertEqual([os.path.join(basedir, 'subdir'),
                          os.path.join(basedir, 'subdir2'),
                          os.path.join(basedir, 'subfile')],
                         sorted(glob.glob('/xyzzy/*')))

    def test_glob_exact(self):
        self.assertEqual(['/xyzzy'], glob.glob('/xyzzy'))
        self.assertEqual(['/xyzzy/subfile'], glob.glob('/xyzzy/subfile'))

    def test_glob_question(self):
        basedir = os.sep + 'xyzzy'
        self.assertEqual([os.path.join(basedir, 'subdir'),
                          os.path.join(basedir, 'subdir2'),
                          os.path.join(basedir, 'subfile')],
                         sorted(glob.glob('/x?zz?/*')))

    def test_glob_no_magic(self):
        self.assertEqual(['/xyzzy'], glob.glob('/xyzzy'))
        self.assertEqual(['/xyzzy/subdir'], glob.glob('/xyzzy/subdir'))

    def test_non_existent_path(self):
        self.assertEqual([], glob.glob('nonexistent'))

    def test_magic_dir(self):
        self.assertEqual([os.sep + '[Temp]'], glob.glob('/*emp*'))

    def test_glob1(self):
        self.assertEqual(['[Temp]'], glob.glob1('/', '*Tem*'))

    def test_has_magic(self):
        self.assertTrue(glob.has_magic('['))
        self.assertFalse(glob.has_magic('a'))


if __name__ == '__main__':
    unittest.main()
